Date: {{ date|safe }}
From: {{ from|safe }}
To: {{ to|safe }}
Message-ID: {{ messageid|safe }}
Subject: {{ subject_header_value|safe }}
MIME-Version: 1.0
Content-Type: multipart/alternative;
	boundary="{{ multipart_boundary|safe }}"

--{{ multipart_boundary|safe }}
Content-Type: text/plain; charset=UTF-8
Content-Disposition: inline

Hi,

{{ sender_display_name|safe }} {{ bracketed_verified_sender|safe }}has invited you into a {% if room_type == "m.space" %}space{% else %}room{% endif %}
{{ bracketed_room_name|safe }}on Element. To join the conversation please follow the
link below.

{{ web_client_location }}/#/room/{{ room_id|urlencode }}?email={{ to|urlencode }}&signurl=https%3A%2F%2Fvector.im%2F_matrix%2Fidentity%2Fapi%2Fv1%2Fsign-ed25519%3Ftoken%3D{{ token|urlencode }}%26private_key%3D{{ ephemeral_private_key|urlencode }}&room_name={{ room_name|urlencode }}&room_avatar_url={{ room_avatar_url|urlencode }}&inviter_name={{ sender_display_name|urlencode }}&guest_access_token={{ guest_access_token|urlencode }}&guest_user_id={{ guest_user_id|urlencode }}&room_type={{ room_type|urlencode }}

Element is an open source collaboration app built on the Matrix.org
open standard for interoperable communication: supporting group chat,
file transfer, voice and video calling, integrations to other apps, bridges
to other communication systems and much more.

Please note that you will need to use Chrome, Firefox or Safari on the web, or
iOS or Android on mobile.

Thanks,

Element


About Element:

Break through - Element allows teams to communicate across a wide range of collaboration
apps. If some team members use Element while others use IRC, Slack or Gitter, Element will
allow these team members to seamlessly work together. Element offers the richest
network of communication bridges.

Own Your Own Data - No one should control your communication and data but you. Element
lets you run your own server, and provides users and teams with the most advanced
crypto ratchet technology available today for a decentralized secure Internet.

Open Source - Element is entirely open source: all the code is published on GitHub
(Apache License) for anyone to see and extend. This means teams can customize or
contribute to the code and everyone can benefit from the speed of community innovation.

Made on Matrix - Element is built on top of Matrix. Matrix is an open network for secure,
decentralized communication delivering a community of users, bridged networks,
integrated bots and applications plus full end-to-end encryption. To learn more about
Matrix visit https://matrix.org.

--{{ multipart_boundary|safe }}
Content-Type: text/html; charset=UTF-8
Content-Disposition: inline

<!doctype html>
<html lang="en">
    <head>
        <style type="text/css">
body {
    margin: 0px;
}

pre, code {
    word-break: break-word;
    white-space: pre-wrap;
}

#page {
    font-family: 'Open Sans', Helvetica, Arial, Sans-Serif;
    font-color: #454545;
    font-size: 12pt;
    width: 100%%;
    padding: 20px;
}

.low-contrast {
    color: #666666
}

#inner {
    width: 640px;
}

.header {
    width: 100%%;
    height: 87px;
    color: #454545;
    border-bottom: 4px solid #e5e5e5;
}

.logo {
    text-align: right;
    margin-left: 20px;
}

.header {
    border-bottom: 4px solid #e4f7ed ! important;
}

.notif_link a, .footer a {
    color: #76CFA6 ! important;
}
        </style>
    </head>
    <body>
        <table id="page">
            <tr>
                <td> </td>
                <td id="inner">
                    <table class="header">
                        <tr>
                            <td>
                            </td>
                            <td class="logo">
                                    <img src="https://static.element.io/images/email-logo.png" width="64" height="64" alt="Element"/>
                            </td>
                        </tr>
                    </table>

<p>Hi,</p>

<p>{{ sender_display_name }} <span class="low-contrast">{{ bracketed_verified_sender }}</span> has invited you into a
{% if room_type == "m.space" %}space{% else %}room{% endif %} {{ bracketed_room_name }} on Element.</p>

<p>
    <a
    href="{{ web_client_location }}/#/room/{{ room_id|urlencode }}?email={{ to|urlencode }}&signurl=https%3A%2F%2Fvector.im%2F_matrix%2Fidentity%2Fapi%2Fv1%2Fsign-ed25519%3Ftoken%3D{{ token|urlencode }}%26private_key%3D{{ ephemeral_private_key|urlencode }}&room_name={{ room_name|urlencode }}&room_avatar_url={{ room_avatar_url|urlencode }}&inviter_name={{ sender_display_name|urlencode }}&guest_access_token={{ guest_access_token|urlencode }}&guest_user_id={{ guest_user_id|urlencode }}&room_type={{ room_type|urlencode }}">Join the conversation.</a>
</p>

<p>Element is an open source collaboration app built on the Matrix.org
open standard for interoperable communication: supporting group chat, file
transfer, voice and video calling, integrations to other apps, bridges
to other communication systems and much more.</p>

<p>Please note that Element requires
<a href="https://www.google.com/chrome">Chrome</a>,
<a href="https://www.getfirefox.com">Firefox</a> or
<a href="https://www.apple.com/safari">Safari</a> on the web,
or iOS or Android on mobile.</p>

<p>Thanks,</p>

<p>Element</p>

<br>
<p>About Element:</p>

<p><em>Break through</em> - Element allows teams to communicate across a wide range of collaboration
apps. If some team members use Element while others use IRC, Slack or Gitter, Element will
allow these team members to seamlessly work together. Element offers the richest
network of communication bridges.</p>

<p><em>Own Your Own Data</em> - No one should control your communication and data but you. Element
lets you run your own server, and provides users and teams with the most advanced
crypto ratchet technology available today for a decentralized secure Internet.</p>

<p><em>Open Source</em> - Element is entirely open source: all the code is published on GitHub
(Apache License) for anyone to see and extend. This means teams can customize or
contribute to the code and everyone can benefit from the speed of community innovation.</p>

<p><em>Made on Matrix</em> - Element is built on top of Matrix. Matrix is an open network for secure,
decentralized communication delivering a community of users, bridged networks,
integrated bots and applications plus full end-to-end encryption. To learn more about
Matrix visit https://matrix.org.</p>

                </td>
                <td> </td>
            </tr>
        </table>
    </body>
</html>

--{{ multipart_boundary|safe }}--
