Date: {{ date|safe }}
From: {{ from|safe }}
To: {{ to|safe }}
Message-ID: {{ messageid|safe }}
Subject: Confirm your email address for Element
MIME-Version: 1.0
Content-Type: multipart/alternative;
	boundary="{{ multipart_boundary|safe }}"

--{{ multipart_boundary|safe }}
Content-Type: text/plain; charset=UTF-8
Content-Disposition: inline

Hello there!

You have asked us to register this email address with element.io - the open source,
distributed and secure shared workspace for the web that's built on Matrix.

If it was really you who made this request, you can click on the following link to
complete the verification of your email address:

{{ link|safe }}

Please note that you will need to use Chrome, Firefox or Safari on the web, or
iOS or Android on mobile.

If you didn't make this request, you can safely disregard this email.

Thanks!

Element


About Element:

Break through - Element allows teams to communicate across a wide range of collaboration
apps. If some team members use Element while others use IRC, Slack or Gitter, Element will
allow these team members to seamlessly work together. Element offers the richest
network of communication bridges.

Own Your Own Data - No one should control your communication and data but you. Element
lets you run your own server, and provides users and teams with the most advanced
crypto ratchet technology available today for a decentralized secure Internet.

Open Source - Element is entirely open source: all the code is published on GitHub
(Apache License) for anyone to see and extend. This means teams can customize or
contribute to the code and everyone can benefit from the speed of community innovation.

Made on Matrix - Element is built on top of Matrix. Matrix is an open network for secure,
decentralized communication delivering a community of users, bridged networks,
integrated bots and applications plus full end-to-end encryption. To learn more about
Matrix visit https://matrix.org.

--{{ multipart_boundary|safe }}
Content-Type: text/html; charset=UTF-8
Content-Disposition: inline

<!doctype html>
<html lang="en">
    <head>
        <style type="text/css">
body {
    margin: 0px;
}

pre, code {
    word-break: break-word;
    white-space: pre-wrap;
}

#page {
    font-family: 'Open Sans', Helvetica, Arial, Sans-Serif;
    font-color: #454545;
    font-size: 12pt;
    width: 100%%;
    padding: 20px;
}

#inner {
    width: 640px;
}

.header {
    width: 100%%;
    height: 87px;
    color: #454545;
    border-bottom: 4px solid #e5e5e5;
}

.logo {
    text-align: right;
    margin-left: 20px;
}

.header {
    border-bottom: 4px solid #e4f7ed ! important;
}


.notif_link a, .footer a {
    color: #76CFA6 ! important;
}
        </style>
    </head>
    <body>
        <table id="page">
            <tr>
                <td> </td>
                <td id="inner">
                    <table class="header">
                        <tr>
                            <td>
                            </td>
                            <td class="logo">
                                    <img src="https://static.element.io/images/email-logo.png" width="64" height="64" alt="Element"/>
                            </td>
                        </tr>
                    </table>

                    <p>Hello there!</p>

                    <p>You have asked us to register this email address with element.io - the open source,
                    distributed and secure shared workspace for the web that's built on Matrix.</p>

                    <p>If it was really you who made this request, you can click on the following link to
                    complete the verification of your email address:</p>

                    <p><a href="{{ link }}">Complete email verification</a></p>


                    <p>Please note that Element requires
                    <a href="https://www.google.com/chrome">Chrome</a>,
                    <a href="https://www.getfirefox.com">Firefox</a> or
                    <a href="https://www.apple.com/safari">Safari</a> on the web,
                    or iOS or Android on mobile.</p>

                    <p>If you didn't make this request, you can safely disregard this email.</p>

                    <p>Thanks!</p>

                    <p>Element</p>

                    <br>
                    <p>About Element:</p>

                    <p><em>Break through</em> - Element allows teams to communicate across a wide range of collaboration
                    apps. If some team members use Element while others use IRC, Slack or Gitter, Element will
                    allow these team members to seamlessly work together. Element offers the richest
                    network of communication bridges.</p>

                    <p><em>Own Your Own Data</em> - No one should control your communication and data but you. Element
                    lets you run your own server, and provides users and teams with the most advanced
                    crypto ratchet technology available today for a decentralized secure Internet.</p>

                    <p><em>Open Source</em> - Element is entirely open source: all the code is published on GitHub
                    (Apache License) for anyone to see and extend. This means teams can customize or
                    contribute to the code and everyone can benefit from the speed of community innovation.</p>

                    <p><em>Made on Matrix</em> - Element is built on top of Matrix. Matrix is an open network for secure,
                    decentralized communication delivering a community of users, bridged networks,
                    integrated bots and applications plus full end-to-end encryption. To learn more about
                    Matrix visit https://matrix.org.</p>

                </td>
                <td> </td>
            </tr>
        </table>
    </body>
</html>

--{{ multipart_boundary|safe }}--
