{
	"token": "{{ token }}",
	"room_alias": "{{ room_alias }}",
	"room_avatar_url": "{{ room_avatar_url }}",
	"room_name": "{{ room_name }}",
	"sender_display_name": "{{ sender_display_name }}",
	"sender_avatar_url": "{{ sender_avatar_url }}"
}
