Date: {{ date|safe }}
From: {{ from|safe }}
To: {{ to|safe }}
Message-ID: {{ messageid|safe }}
Subject: We have changed the way your Matrix account and email address are associated
MIME-Version: 1.0
Content-Type: multipart/alternative;
	boundary="{{ multipart_boundary|safe }}"

--{{ multipart_boundary|safe }}
Content-Type: text/plain; charset=UTF-8
Content-Disposition: inline

Hello,

We’ve recently improved how people discover your Matrix account.
In the past, identity services took capitalisation into account when storing email
addresses. This means Alice@example.com and alice@example.com would be considered to be
two different addresses, and could be associated with different Matrix accounts. We’ve
now updated this behaviour so anyone can find you, no matter how your email is capitalised.
As part of this recent update, we've dissociated the Matrix account {{ mxid|safe }} from
this e-mail address.
No action is needed on your part. This doesn’t affect any passwords or password reset
options on your account.

Thanks,

Element


About Element:

Break through - Element allows teams to communicate across a wide range of collaboration
apps. If some team members use Element while others use IRC, Slack or Gitter, Element will
allow these team members to seamlessly work together. Element offers the richest
network of communication bridges.

Own Your Own Data - No one should control your communication and data but you. Element
lets you run your own server, and provides users and teams with the most advanced
crypto ratchet technology available today for a decentralized secure Internet.

Open Source - Element is entirely open source: all the code is published on GitHub
(Apache License) for anyone to see and extend. This means teams can customize or
contribute to the code and everyone can benefit from the speed of community innovation.

Made on Matrix - Element is built on top of Matrix. Matrix is an open network for secure,
decentralized communication delivering a community of users, bridged networks,
integrated bots and applications plus full end-to-end encryption. To learn more about
Matrix visit https://matrix.org.

--{{ multipart_boundary|safe }}
Content-Type: text/html; charset=UTF-8
Content-Disposition: inline

<!doctype html>
<html lang="en">
    <head>
        <style type="text/css">
body {
    margin: 0px;
}

pre, code {
    word-break: break-word;
    white-space: pre-wrap;
}

#page {
    font-family: 'Open Sans', Helvetica, Arial, Sans-Serif;
    font-color: #454545;
    font-size: 12pt;
    width: 100%%;
    padding: 20px;
}

.low-contrast {
    color: #666666
}

#inner {
    width: 640px;
}

.header {
    width: 100%%;
    height: 87px;
    color: #454545;
    border-bottom: 4px solid #e5e5e5;
}

.logo {
    text-align: right;
    margin-left: 20px;
}

.header {
    border-bottom: 4px solid #e4f7ed ! important;
}

.notif_link a, .footer a {
    color: #76CFA6 ! important;
}
        </style>
    </head>
    <body>
        <table id="page">
            <tr>
                <td> </td>
                <td id="inner">
                    <table class="header">
                        <tr>
                            <td>
                            </td>
                            <td class="logo">
                                    <img src="https://static.element.io/images/email-logo.png" width="64" height="64" alt="Element"/>
                            </td>
                        </tr>
                    </table>

<p>Hello,</p>

<p>
    We’ve recently improved how people discover your Matrix account.<br/>
    In the past, identity services took capitalisation into account when storing email
    addresses. This means Alice@example.com and alice@example.com would be considered to
    be two different addresses, and could be associated with different Matrix accounts.
    We’ve now updated this behaviour so anyone can find you, no matter how your email is
    capitalised. As part of this recent update, we've dissociated the Matrix account
    {{ mxid|safe }} from this e-mail address.<br/>
    No action is needed on your part. This doesn’t affect any passwords or password reset
    options on your account.
</p>

<p>Thanks,</p>

<p>Element</p>

<br>
<p>About Element:</p>

<p><em>Break through</em> - Element allows teams to communicate across a wide range of collaboration
apps. If some team members use Element while others use IRC, Slack or Gitter, Element will
allow these team members to seamlessly work together. Element offers the richest
network of communication bridges.</p>

<p><em>Own Your Own Data</em> - No one should control your communication and data but you. Element
lets you run your own server, and provides users and teams with the most advanced
crypto ratchet technology available today for a decentralized secure Internet.</p>

<p><em>Open Source</em> - Element is entirely open source: all the code is published on GitHub
(Apache License) for anyone to see and extend. This means teams can customize or
contribute to the code and everyone can benefit from the speed of community innovation.</p>

<p><em>Made on Matrix</em> - Element is built on top of Matrix. Matrix is an open network for secure,
decentralized communication delivering a community of users, bridged networks,
integrated bots and applications plus full end-to-end encryption. To learn more about
Matrix visit https://matrix.org.</p>

                </td>
                <td> </td>
            </tr>
        </table>
    </body>
</html>

--{{ multipart_boundary|safe }}--
